==> hermes-5.6.0.2.tar.gz: hermes-5.6.0.2/spec/jobs/hermes/log_job_spec.rb <==
require 'rails_helper'
require 'thread'
require 'thwait'

RSpec.describe Hermes::LogJob do
  subject { Hermes::LogJob }
  let(:redis_key) { 'redis_hermes_logs_queue' }

  before(:each) do
    $redis.del(redis_key)
    drain_all_jobs
  end

  it 'should set logs in queue' do
    expect do
      subject.add_to_queue(code: '1234', action: 'test')
    end.to change { Hermes::LogJob.queue_size }.by 1
  end

  it 'should no create log if queue is not full' do
    5.times do
      subject.add_to_queue(code: '1234', action: 'test')
    end
    drain_all_jobs
    expect(Hermes::LogJob.queue_size).to eq(5)
    expect(Log.count).to eq(0)
  end

  it 'should create log if queue is full' do
    (Hermes::LogJob.queue_max_size).times do
      subject.add_to_queue(code: '1234', action: 'test')
    end
    drain_all_jobs
    expect(Log.count).to eq(Hermes::LogJob.queue_max_size)
  end

  it 'should use default values' do
    subject.add_to_queue(code: '1234', action: 'test')
    first_elem =  JSON.parse($redis.blpop(redis_key)[1])
    expect(first_elem['level']).to eq('info')
    expect(first_elem['domain']).to eq('api')
  end

  it 'should surchage default values' do
    subject.add_to_queue(code: '1234', action: 'test', level: 'debug', domain: 'lol')
    first_elem =  JSON.parse($redis.blpop(redis_key)[1])
    expect(first_elem['level']).to eq('debug')
    expect(first_elem['domain']).to eq('lol')
  end

  it 'Log model created' do 
    (Hermes::LogJob.queue_max_size + 5).times do
      subject.add_to_queue({code: '1234', action: 'test'}, {force_perform_now: true})
    end
    expect(Log.where(code: '1234').count).to eq(Hermes::LogJob.queue_max_size)
  end 

  describe 'concurrency calls' do 
    it 'works' do 
      number_of_threads = 10
      job_per_threads = Hermes::LogJob.queue_max_size * 5

      threads = number_of_threads.times.map do |i|
        Thread.new do
          job_per_threads.times do |j|
            Hermes::LogJob.add_to_queue({
                user_id: "test", 
                code: "API-TEST-PERF", 
                action:"test perform #{i}/#{j}",
              }, 
              {force_perform_now: true}
            )
          end 
        end
      end

      ThreadsWait.all_waits(*threads)

      expect(Hermes::LogJob.queue_size > 0).to eq(true)
      expect(Hermes::LogJob.queue_size < Hermes::LogJob.queue_max_size).to eq(true)
      expect(Log.where(code:  "API-TEST-PERF").count + Hermes::LogJob.queue_size).to eq(number_of_threads * job_per_threads)
    end 
  end 
end

==> hermes-5.6.0.18.tar.gz: hermes-5.6.0.18/spec/jobs/hermes/log_job_spec.rb <==
require 'rails_helper'
require 'thread'
require 'thwait'

RSpec.describe Hermes::LogJob do
  subject { Hermes::LogJob }
  let(:redis_key) { 'redis_hermes_logs_queue' }

  before(:each) do
    $redis.del(redis_key)
    drain_all_jobs
  end

  it 'should set logs in queue' do
    expect do
      subject.add_to_queue(code: '1234', action: 'test')
    end.to change { Hermes::LogJob.queue_size }.by 1
  end

  it 'should no create log if queue is not full' do
    5.times do
      subject.add_to_queue(code: '1234', action: 'test')
    end
    drain_all_jobs
    expect(Hermes::LogJob.queue_size).to eq(5)
    expect(Log.count).to eq(0)
  end

  it 'should create log if queue is full' do
    (Hermes::LogJob.queue_max_size).times do
      subject.add_to_queue(code: '1234', action: 'test')
    end
    drain_all_jobs
    expect(Log.count).to eq(Hermes::LogJob.queue_max_size)
  end

  it 'should use default values' do
    subject.add_to_queue(code: '1234', action: 'test')
    first_elem =  JSON.parse($redis.blpop(redis_key)[1])
    expect(first_elem['level']).to eq('info')
    expect(first_elem['domain']).to eq('api')
  end

  it 'should surchage default values' do
    subject.add_to_queue(code: '1234', action: 'test', level: 'debug', domain: 'lol')
    first_elem =  JSON.parse($redis.blpop(redis_key)[1])
    expect(first_elem['level']).to eq('debug')
    expect(first_elem['domain']).to eq('lol')
  end

  it 'Log model created' do 
    created_at = DateTime.current + 1.days
    (Hermes::LogJob.queue_max_size + 5).times do
      subject.add_to_queue({code: '1234', action: 'test', created_at: created_at}, {force_perform_now: true})
    end
    expect(Log.where(code: '1234').count).to eq(Hermes::LogJob.queue_max_size)
    log = Log.find_by(code: '1234')
    expect(log.created_at.to_date).to eq(created_at.to_date)
    expect(log.updated_at.to_date).to eq(created_at.to_date)
  end 

  describe 'concurrency calls' do 
    it 'works' do 
      number_of_threads = 10
      job_per_threads = Hermes::LogJob.queue_max_size * 5

      threads = number_of_threads.times.map do |i|
        Thread.new do
          job_per_threads.times do |j|
            Hermes::LogJob.add_to_queue({
                user_id: "test", 
                code: "API-TEST-PERF", 
                action:"test perform #{i}/#{j}",
              }, 
              {force_perform_now: true}
            )
          end 
        end
      end

      ThreadsWait.all_waits(*threads)

      expect(Hermes::LogJob.queue_size > 0).to eq(true)
      expect(Hermes::LogJob.queue_size < Hermes::LogJob.queue_max_size).to eq(true)
      expect(Log.where(code:  "API-TEST-PERF").count + Hermes::LogJob.queue_size).to eq(number_of_threads * job_per_threads)
    end 
  end 
end
